<?php
/**
 * 2007-2022 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    PrestaShop SA <contact@prestashop.com>
 *  @copyright 2007-2022 PrestaShop SA
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */
if (!defined('_PS_VERSION_')) {
    exit;
}
require_once(_PS_MODULE_DIR_ . "addifyregistrationfields/classes/AddifyregistrationfieldsModel.php");
require_once(_PS_MODULE_DIR_ . "addifyregistrationfields/classes/AddifyregistrationfieldsdataModel.php");
require_once(_PS_MODULE_DIR_ . "addifyregistrationfields/classes/Addifyregistrationheading.php");
class Addifyregistrationfields extends Module
{
    public function __construct()
    {
        $this->name = 'addifyregistrationfields';
        $this->tab = 'front_office_features';
        $this->version = '1.1.0';
        $this->author = 'Addify';
        $this->need_instance = 0;
        $this->module_key = '599f25952fa0dcedc082777cb9ab4155';
        /**
         * Set $this->bootstrap to true if your module is compliant with bootstrap (PrestaShop 1.6)
         */
        $this->bootstrap = true;
        parent::__construct();
        $this->displayName = $this->l('Addify Custom Registration Fields');
        $this->description = $this->l('Add an unlimited number of custom fields to your registration form to gather additional information seamlessly during the registration process.');
        $this->confirmUninstall = $this->l('Are you sure you want to uninstall my module?');
        $this->ps_versions_compliancy = array('min' => '1.7', 'max' => _PS_VERSION_);
    }
    public function install()
    {
        include(dirname(__FILE__) . '/sql/install.php');
        return parent::install() &&
            $this->registerHook('displayHeader')
            && $this->registerHook('displayBackOfficeHeader')
            && $this->registerHook('displayCustomerAccountForm')
            && $this->registerHook('actionCustomerAccountAdd')
            && $this->registerHook('actionSubmitAccountBefore')
            && $this->registerHook('displayAdminCustomers')
            && $this->registerHook('registerGDPRConsent')
            && $this->registerHook('actionDeleteGDPRCustomer')
            && $this->registerHook('actionExportGDPRData')
            // && $this->registerHook('actionObjectCustomerUpdateBefore')
            // && $this->registerHook('actionCustomerAccountUpdate')
            && $this->registerHook('displayGDPRConsent');
    }
    public function uninstall()
    {
        include(dirname(__FILE__) . '/sql/uninstall.php');
        Configuration::deleteByName('addifyregistrationfields_LIVE_MODE');
        Configuration::deleteByName('addifyregistrationfields_show_field_account');
        Configuration::deleteByName('addifyregistrationfields_show_field_admin');
        return parent::uninstall() &&
            $this->unregisterHook('displayHeader')
            && $this->unregisterHook('displayBackOfficeHeader')
            && $this->unregisterHook('displayCustomerAccountForm')
            && $this->unregisterHook('actionCustomerAccountAdd')
            && $this->unregisterHook('actionSubmitAccountBefore')
            && $this->unregisterHook('displayAdminCustomers')
            && $this->unregisterHook('registerGDPRConsent')
            && $this->unregisterHook('actionDeleteGDPRCustomer')
            && $this->unregisterHook('actionExportGDPRData')
            // && $this->unregisterHook('actionObjectCustomerUpdateBefore')
            // && $this->unregisterHook('actionCustomerAccountUpdate')
            && $this->unregisterHook('displayGDPRConsent');
    }
    /**
     * Load the form
     */
    public function getContent()
    {
        if (!$this->active) {
            $this->context->controller->errors[] = $this->l('Access Denied...Module is disabled.');
        } else {
            $active_tab = Tools::getValue('active_tab');
            if (!$active_tab) {
                $active_tab = 1;
            }
            if (((bool) Tools::isSubmit('addfieldsaddifyregisterationfields')) == true) {
                return $this->renderForm();
            }
            if (((bool) Tools::isSubmit('updateaddifyregisterationfields')) == true) {
                return $this->renderForm();
            }
            if (((bool) Tools::isSubmit('addheadingaddifyregistrationfields')) == true || ((bool) Tools::isSubmit('updateaddifyregistrationfieldsheading')) == true) {
                return $this->renderHeadingForm();
            }
            $this->postProcess();
            $data = AddifyregistrationfieldsModel::getPosts();
            $this->context->smarty->assign(
                array(
                    'render_confriguration_form' => $this->getConfigForm(),
                    'render_list' => $this->renderHeadingList(),
                    'active_tab' => $active_tab,
                    'configUrl' => $this->context->link->getAdminLink('AdminModules', true) . '&configure=' . $this->name,
                    'additionalregisterationfieldlist' => $data,
                    'adminToken' => Tools::getAdminTokenLite('AdminModules'),
                    'admincontrollerlink' => $this->context->link->getAdminLink('Adminaddifyregisterationfieldsorting'),
                )
            );
            // return $this->getConfigForm();
            return $this->display(__FILE__, 'views/templates/admin/configure.tpl');
        }
    }
    protected function postProcess()
    {
        if (Tools::isSubmit('Submitconfig' . $this->name)) {
            $this->processConfigSave();
        }
        if (Tools::isSubmit('Submitpost' . $this->name)) {
            $this->processSave();
        }
        if (Tools::isSubmit('Submitheading' . $this->name)) {
            $this->processheadingSave();
        }
        if (Tools::isSubmit('statusaddifyregisterationfields')) {
            if (!AddifyregistrationfieldsModel::setStatus('rule_active', (int) Tools::getValue('id_field'))) {
                $this->context->controller->errors[] = $this->l('Status update failed.');
            } else {
                $this->flushCache();
                Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&conf=5&active_tab=2&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name);
                // $this->context->controller->confirmations[] = $this->l('Status updated successfully.');
            }
        }
        if (Tools::isSubmit('requirmentaddifyregisterationfields')) {
            if (!AddifyregistrationfieldsModel::setRequiredStatus('values_req', (int) Tools::getValue('id_field'))) {
                $this->context->controller->errors[] = $this->l('Required Status update failed.');
            } else {
                $this->flushCache();
                Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&conf=5&active_tab=2&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name);
                // $this->context->controller->confirmations[] = $this->l('Required Status updated successfully.');
            }
        }
        if (Tools::isSubmit('deleteaddifyregisterationfields')) {
            if (!Validate::isLoadedObject($id = new AddifyregistrationfieldsModel((int) Tools::getValue('id')))) {
            } else {
                if (!$id->delete()) {
                    $this->context->controller->errors[] = $this->l('Rule cannot be deleted.');
                } else {
                    $this->context->controller->confirmations[] = $this->l('Rule deleted successfully.');
                }
            }
        }
        if (Tools::isSubmit('deleteaddifyregistrationfieldsheading')) {
            if (!Validate::isLoadedObject($id_heading = new AddifyregistrationheadingModel((int) Tools::getValue('id_heading')))) {
            } else {
                if (!$id_heading->delete()) {
                    Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&conf=4&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name . '&active_tab=2');
                } else {
                    $this->context->controller->confirmations[] = $this->l('Rule deleted successfully.');
                }
            }
        }
        if (Tools::isSubmit('submitBulkdelete' . $this->name)) {
            $postBox = Tools::getValue('addifyregistrationfieldsBox');
            if (isset($postBox) && $postBox) {
                $deleted = 0;
                foreach ($postBox as $posts) {
                    if (!Validate::isLoadedObject($AddifyregistrationfieldsModel = new AddifyregistrationfieldsModel((int) $posts))) {
                        break;
                    } else {
                        if ($AddifyregistrationfieldsModel->delete()) {
                            $deleted++;
                        }
                    }
                }
                if (count($this->context->controller->errors)) {
                    $this->context->controller->errors;
                } else {
                    $this->context->controller->confirmations[] = sprintf($this->l('Rules deleted successfully.'), $deleted);
                }
            }
        }
    }
    /**
     * Create the form that will be displayed in the configuration of your module.
     */
    protected function renderHeadingForm()
    {
        $fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('Headings'),
                    'icon' => 'icon-cogs'
                ),
                'input' => array(
                    array(
                        'type' => 'hidden',
                        'name' => 'id_heading'
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Heading'),
                        'name' => 'addifyregistrationfields_heading',
                        'desc' => $this->l('Enable/Disable this Heading'),
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->l('Enable')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('Disable')
                            ),
                        ),
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('Heading Text'),
                        'name' => 'heading_text',
                        'desc' => $this->l('Enter Heading for rule'),
                    ),
                ),
                'submit' => [
                    'title' => $this->l('Save'),
                ],
            )
        );
        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'Submitheading' . $this->name;
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false)
            . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = array(
            'fields_value' => $this->getHeadingFormValues(),  /* Add values for your inputs */
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id
        );
        return $helper->generateForm([$fields_form]);
    }
    protected function getHeadingFormValues()
    {
        $fields_value = array();
        if (Tools::getValue('id_heading') && (Tools::isSubmit('Submitheading' . $this->name) === false)) {
            $postBlock = new AddifyregistrationheadingModel((int) Tools::getValue('id_heading'));
            $fields_value['id_heading'] = $postBlock->id_heading;
            $fields_value['addifyregistrationfields_heading'] = $postBlock->heading_active;
            $fields_value['heading_text'] = $postBlock->heading_text;
        } else {
            $fields_value['id_heading'] = Tools::getValue('id_heading');
            $fields_value['addifyregistrationfields_heading'] = Tools::getValue('addifyregistrationfields_heading');
            $fields_value['heading_text'] = Tools::getValue('heading_text');
        }
        return $fields_value;
    }
    protected function processheadingSave()
    {
        $call_back = 'add';
        $form_values = $this->getHeadingFormValues();
        if (!isset($form_values) && !$form_values) {
            return $this->context->controller->errors[] = $this->l('Empty post values.');
        }
        if ($form_values['addifyregistrationfields_heading'] < 0) {
            $this->context->controller->errors[] = $this->l('Heading value is not valid.');
        }
        if (empty($form_values['heading_text'])) {
            $this->context->controller->errors[] = $this->l('Heading Text must be required');
        }
        if (count($this->context->controller->errors)) {
            return $this->context->controller->errors;
        }
        if (($id_heading = Tools::getValue('id_heading'))) {
            $call_back = 'update';
            $AddifyregistrationheadingModel = new AddifyregistrationheadingModel((int) $id_heading);
            $AddifyregistrationheadingModel->heading_active = $form_values['addifyregistrationfields_heading'];
            $AddifyregistrationheadingModel->heading_text = $form_values['heading_text'];
        } else {
            $AddifyregistrationheadingModel = new AddifyregistrationheadingModel();
            $AddifyregistrationheadingModel->heading_active = $form_values['addifyregistrationfields_heading'];
            $AddifyregistrationheadingModel->heading_text = $form_values['heading_text'];
        }
        if (!call_user_func(array($AddifyregistrationheadingModel, $call_back))) {
            $this->context->controller->errors = sprintf($this->l('Something went wrong while performing operation %s'), $call_back);
        } else {
            $this->flushCache();
            if ($call_back == 'add') {
                Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&conf=3&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name . '&active_tab=2');
            } else {
                Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&conf=4&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name . '&active_tab=2');
            }
        }
    }
    protected function renderHeadingList()
    {
        $fields_list = array(
            'id_heading' => array(
                'title' => $this->l('ID'),
                'width' => 120,
                'type' => 'int',
                'search' => false,
                'orderby' => false
            ),
            'heading_text' => array(
                'title' => $this->l('Title'),
                'width' => 140,
                'type' => 'text',
                'search' => false,
                'orderby' => false
            ),
            'heading_active' => array(
                'title' => $this->l('Status'),
                'width' => 140,
                'active' => 'status',
                'type' => 'bool',
                'search' => false,
                'orderby' => false,
            ),
            'date_add' => array(
                'title' => $this->l('Added Date'),
                'width' => 140,
                'type' => 'text',
                'search' => false,
                'orderby' => false,
            ),
            'date_update' => array(
                'title' => $this->l('Updated Date'),
                'width' => 140,
                'type' => 'text',
                'search' => false,
                'orderby' => false,
            ),
        );
        $helper = new HelperList();
        $helper->shopLinkType = '';
        $helper->simple_header = false;
        $helper->identifier = 'id_heading';
        $helper->actions = array('edit', 'delete');
        $helper->bulk_actions = array(
            'delete' => array(
                'text' => $this->l('Delete selected'),
                'icon' => 'icon-trash',
                'confirm' => $this->l('Delete selected items?')
            )
        );
        $helper->listTotal = count(AddifyregistrationheadingModel::getlist());
        $helper->show_toolbar = true;
        $helper->toolbar_btn['new'] = array(
            'href' => AdminController::$currentIndex . '&configure=' . $this->name . '&addheading' . $this->name . '&token=' . Tools::getAdminTokenLite('AdminModules'),
            'desc' => $this->l('Add new rule')
        );
        $helper->title = "Heading";
        $helper->table = "addifyregistrationfieldsheading";
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex . '&configure=' . $this->name;
        return $helper->generateList(AddifyregistrationheadingModel::getlist(), $fields_list);
    }
    /**
     * Create the structure of your form.
     */
    protected function renderForm()
    {
        // $headinglists = AddifyregistrationheadingModel::getlistactive();
        // array_unshift($headinglists, array('id_heading' => 0, 'heading_text' => 'Select the option'));
        $fields_dependent = array(
            array('id' => 'none', 'name' => $this->l('None'))
        );
        $get_rules = AddifyregistrationfieldsModel::getRulesForFields();
        if (Tools::getValue('id')) {
            $dependent_value = array();
            $get_saved_data = new AddifyregistrationfieldsModel((int) Tools::getValue('id'));
            foreach ($get_rules as $get_rule) {
                if ($get_rule['field_type'] == "multiselect" || $get_rule['field_type'] == "select" || $get_rule['field_type'] == "checkbox" || $get_rule['field_type'] == "radio") {
                    if ($get_saved_data->id != $get_rule['id']) {
                        $fields_dependent[] = array('id' => $get_rule['id'], 'name' => $get_rule['id'] . '-' . $get_rule['field_name']);
                    }
                }
            }
        } else {
            $dependent_value = array(
                array('id' => 'none', 'name' => $this->l('None')),
            );
            foreach ($get_rules as $get_rule) {
                if ($get_rule['field_type'] == "multiselect" || $get_rule['field_type'] == "select" || $get_rule['field_type'] == "checkbox" || $get_rule['field_type'] == "radio") {
                    $fields_dependent[] = array('id' => $get_rule['id'], 'name' => $get_rule['id'] . '-' . $get_rule['field_name']);
                }
            }
        }
        $fields_types = array(
            array('id' => 'text', 'name' => $this->l('Text')),
            array('id' => 'textarea', 'name' => $this->l('Textarea')),
            array('id' => 'number', 'name' => $this->l('Number')),
            array('id' => 'vat_number', 'name' => $this->l('VAT Number')),
            array('id' => 'email', 'name' => $this->l('Email')),
            array('id' => 'select', 'name' => $this->l('Select')),
            array('id' => 'multiselect', 'name' => $this->l('MultiSelect')),
            array('id' => 'checkbox', 'name' => $this->l('Checkbox(s)')),
            array('id' => 'radio', 'name' => $this->l('Radio Button(s)')),
            array('id' => 'date', 'name' => $this->l('Date')),
            array('id' => 'switch', 'name' => $this->l('Switch')),
            array('id' => 'fileupload', 'name' => $this->l('Attachment')),
            array('id' => 'time', 'name' => $this->l('Time')),
            array('id' => 'color', 'name' => $this->l('color')),
            array('id' => 'heading', 'name' => $this->l('Heading (display only)')),
            array('id' => 'message', 'name' => $this->l('Message (display only)')),
        );
        $fields_validations = array(
            array('id' => 'none', 'name' => $this->l('None')),
            array('id' => 'isVAT', 'name' => $this->l('VAT Number Validation')),
            array('id' => 'isFloat', 'name' => $this->l('Decimal Number')),
            array('id' => 'isInt', 'name' => $this->l('Integer Number')),
            array('id' => 'isEmail', 'name' => $this->l('Email Address')),
            array('id' => 'isGenericName', 'name' => $this->l('Only Letters')),
            array('id' => 'isString', 'name' => $this->l('Letters, Numbers and Symbols')),
            array('id' => 'isUrl', 'name' => $this->l('Website Url Address')),
            array('id' => 'isDate', 'name' => $this->l('Date')),
        );
        $fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('Fields'),
                    'icon' => 'icon-cogs',
                ),
                'input' => array(
                    array(
                        'type' => 'hidden',
                        'name' => 'id'
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Field'),
                        'name' => 'rule_active',
                        'is_bool' => true,
                        'desc' => $this->l('Enable/disable field from here.'),
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('Field Title'),
                        'name' => 'title',
                        'desc' => $this->l('Add field title here. (This is for back office only)'),
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('Priority'),
                        'name' => 'priority',
                        'validate' => 'isInt',
                        'required' => true,
                        'desc' => $this->l('Value must be numeric.'),
                    ),
                    array(
                        'type' => 'select',
                        'label' => $this->l('Field Type'),
                        'name' => 'field_type',
                        'desc' => $this->l('Select field type here.'),
                        'options' => array(
                            'query' => $fields_types,
                            'id' => 'id',
                            'name' => 'name'
                        ),
                    ),
                    array(
                        'type' => 'select',
                        'label' => $this->l('Field Validation'),
                        'name' => 'field_validation',
                        'desc' => $this->l('If field type "Text" is selected then validation is applicable (Letters, Numbers and Symbols, Only Letters, Website Url Address, none). If field type "Number" is selected then validation is applicable (Decimal Number, Integer Number, none). Other fields types are the same set as field validation.'),
                        'options' => array(
                            'query' => $fields_validations,
                            'id' => 'id',
                            'name' => 'name'
                        ),
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('Field Name'),
                        'name' => 'field_name',
                        'desc' => $this->l('This should be unique for example : "name or middle_name" etc.'),
                        'required' => true,
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('Field Label'),
                        'name' => 'field_label',
                        'required' => true,
                    ),
                    array(
                        'type' => 'textarea',
                        'label' => $this->l('Short Description'),
                        'desc' => $this->l('Add custom description here (this is for back office only'),
                        'name' => 'short_description',
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('Placeholder'),
                        'desc' => $this->l('This text will be displayed inside the field.'),
                        'name' => 'place_holder',
                    ),
                    // array(
                    //     'type' => 'select',
                    //     'label' => $this->l('Heading'),
                    //     'name' => 'field_heading',
                    //     'options' => array(
                    //         'query' => $headinglists,
                    //         'id' => 'heading_text',
                    //         'name' => 'heading_text'
                    //     ),
                    // ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('Options'),
                        'name' => 'options',
                        'placeholder' => 'option_value1->option_label1,option_value2->option_label2,option_value3->option_label3...',
                        'desc' => $this->l('Separate each value and option with -> and all options->values with comma sign.'),
                    ),
                    array(
                        'type' => 'textarea',
                        'label' => $this->l('Message for Display'),
                        'name' => 'display_msg',
                    ),
                    array(
                        'type' => 'textarea',
                        'label' => $this->l('Heading for Display'),
                        'name' => 'field_heading',
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('Maximum size'),
                        'name' => 'max_size',
                        'suffix' => $this->l('MB'),
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('File Types'),
                        'name' => 'file_type',
                        'desc' => $this->l('Enter comma(,) separated values.'),
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Values Required'),
                        'name' => 'values_req',
                        'is_bool' => true,
                        'desc' => $this->l('Enable to make a required field.'),
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Allow users to edit'),
                        'name' => 'users_to_edit',
                        'is_bool' => true,
                        'desc' => $this->l('If enabled then users should be able to change the information they have added after the registration. '),
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Dependent Field'),
                        'name' => 'enable_dependent_field',
                        'is_bool' => true,
                        'desc' => $this->l('Enable to make a dependent field.'),
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),
                    array(
                        'type' => 'select',
                        'label' => $this->l('Select Dependent Field'),
                        'name' => 'dependent_field',
                        'options' => array(
                            'query' => $fields_dependent,
                            'id' => 'id',
                            'name' => 'name'
                        ),
                    ),
                    array(
                        'type' => 'select',
                        'label' => $this->l('Select Dependent Value'),
                        'name' => 'dependent_value',
                        'options' => array(
                            'query' => $dependent_value,
                            'id' => 'id',
                            'name' => 'name'
                        ),
                    ),
                ),
                'submit' => array(
                    'title' => $this->l('Save'),
                ),
                'buttons' => array(
                    'cancel' => array(
                        'title' => $this->l('Cancel'),
                        'class' => 'btn btn-default',
                        'icon' => 'process-icon-cancel',
                        'href' => AdminController::$currentIndex . '&configure=' . $this->name . '&token=' . Tools::getAdminTokenLite('AdminModules'),
                    ),
                )
            ),
        );
        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->show_cancel_button = false;
        $helper->table = 'addifyregistrationfields';
        $helper->module = $this;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);
        $helper->identifier = 'id';
        $helper->submit_action = 'Submitpost' . $this->name;
        $helper->currentIndex = AdminController::$currentIndex . '&configure=' . $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->back_url = $helper->currentIndex . '&token=' . $helper->token;
        $helper->tpl_vars = array(
            'fields_value' => $this->getFormValues(), /* Add values for your inputs */
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
            'context_link' => Context::getContext()->link,
        );
        return $helper->generateForm(array($fields_form));
    }
    protected function getFormValues()
    {
        $fields_value = array();
        if (Tools::getValue('id') && (Tools::isSubmit('Submitpost' . $this->name) === false)) {
            $postBlock = new AddifyregistrationfieldsModel((int) Tools::getValue('id'));
            $fields_value['id'] = $postBlock->id;
            $fields_value['rule_active'] = $postBlock->rule_active;
            $fields_value['title'] = $postBlock->title;
            $fields_value['priority'] = $postBlock->priority;
            $fields_value['max_size'] = $postBlock->max_size;
            $fields_value['file_type'] = $postBlock->file_type;
            $fields_value['field_type'] = $postBlock->field_type;
            $fields_value['dependent_field'] = $postBlock->dependent_field;
            $fields_value['field_validation'] = $postBlock->field_validation;
            $fields_value['field_name'] = $postBlock->field_name;
            $fields_value['field_label'] = $postBlock->field_label;
            $fields_value['short_description'] = $postBlock->short_description;
            $fields_value['place_holder'] = $postBlock->place_holder;
            $fields_value['field_heading'] = $postBlock->field_heading;
            $fields_value['options'] = $postBlock->options;
            $fields_value['values_req'] = $postBlock->values_req;
            $fields_value['users_to_edit'] = $postBlock->users_to_edit;
            $fields_value['display_msg'] = $postBlock->display_msg;
            $fields_value['enable_dependent_field'] = $postBlock->enable_dependent_field;
            $fields_value['dependent_value'] = $postBlock->dependent_value;
        } else {
            $fields_value['id'] = Tools::getValue('id');
            $fields_value['rule_active'] = Tools::getValue('rule_active');
            $fields_value['title'] = Tools::getValue('title');
            $fields_value['priority'] = Tools::getValue('priority');
            $fields_value['max_size'] = Tools::getValue('max_size');
            $fields_value['file_type'] = Tools::getValue('file_type');
            $fields_value['field_type'] = Tools::getValue('field_type');
            $fields_value['dependent_field'] = Tools::getValue('dependent_field');
            $fields_value['field_validation'] = Tools::getValue('field_validation');
            $fields_value['field_name'] = Tools::getValue('field_name');
            $fields_value['field_label'] = Tools::getValue('field_label');
            $fields_value['short_description'] = Tools::getValue('short_description');
            $fields_value['place_holder'] = Tools::getValue('place_holder');
            $fields_value['field_heading'] = Tools::getValue('field_heading');
            $fields_value['options'] = Tools::getValue('options');
            $fields_value['values_req'] = Tools::getValue('values_req');
            $fields_value['users_to_edit'] = Tools::getValue('users_to_edit');
            $fields_value['enable_dependent_field'] = Tools::getValue('enable_dependent_field');
            $fields_value['dependent_value'] = Tools::getValue('dependent_value');
            $fields_value['display_msg'] = Tools::getValue('display_msg');
        }
        return $fields_value;
    }
    protected function processSave()
    {
        $call_back = 'add';
        $form_values = $this->getFormValues();
        if (!isset($form_values) && !$form_values) {
            return $this->context->controller->errors[] = $this->l('Empty post values.');
        }
        if ($form_values['priority'] < 0) {
            $this->context->controller->errors[] = $this->l('Priority value is not valid.');
        }

        if ($form_values['field_type'] == 'message' || $form_values['field_type'] == 'heading') {

        } else {
            if (empty($form_values['field_name'])) {
                $this->context->controller->errors[] = $this->l('Field Name must be required');
            }
            if (empty($form_values['field_label'])) {
                $this->context->controller->errors[] = $this->l('Field Label must be required');
            }
        }
        if (($form_values['priority']) == 0) {
            $this->context->controller->errors[] = $this->l('Priority value is not valid.');
        }
        if (!is_numeric($form_values['priority'])) {
            $this->context->controller->errors[] = $this->l('Priority value is not valid.');
        }
        $field_type = $form_values['field_type'];
        if ($field_type == 'select' || $field_type == 'checkbox' || $field_type == 'radio' || $field_type == 'multiselect') {
            $field_options_str = $form_values['options'];
            $field_option_sep = explode(',', $field_options_str);
            foreach ($field_option_sep as $key) {
                $str_check = strrpos($key, "->");
                if ($str_check == '') {
                    $this->context->controller->errors[] = $this->l('invalid Pattern in Input Options.');
                }
                $options = explode('->', $key);
                if ($options[0] == '') {
                    $this->context->controller->errors[] = $this->l('invalid value in Input Options.');
                } elseif ($options[1] == '') {
                    $this->context->controller->errors[] = $this->l('invalid option in Input Options.');
                }
            }
        }
        if (count($this->context->controller->errors)) {
            return $this->context->controller->errors;
        }
        $dt = date("Y-m-d h:i:s");
        if (($id = Tools::getValue('id'))) {
            $call_back = 'update';
            $AddifyregistrationfieldsModel = new AddifyregistrationfieldsModel((int) $id);
            $AddifyregistrationfieldsModel->rule_active = $form_values['rule_active'];
            $AddifyregistrationfieldsModel->title = $form_values['title'];
            $AddifyregistrationfieldsModel->priority = $form_values['priority'];
            $AddifyregistrationfieldsModel->max_size = $form_values['max_size'];
            $AddifyregistrationfieldsModel->file_type = $form_values['file_type'];
            $AddifyregistrationfieldsModel->field_type = $form_values['field_type'];
            $AddifyregistrationfieldsModel->dependent_field = $form_values['dependent_field'];
            $AddifyregistrationfieldsModel->field_validation = $form_values['field_validation'];
            $AddifyregistrationfieldsModel->field_name = $form_values['field_name'];
            $AddifyregistrationfieldsModel->field_label = $form_values['field_label'];
            $AddifyregistrationfieldsModel->short_description = $form_values['short_description'];
            $AddifyregistrationfieldsModel->place_holder = $form_values['place_holder'];
            $AddifyregistrationfieldsModel->field_heading = $form_values['field_heading'];
            $AddifyregistrationfieldsModel->options = $form_values['options'];
            $AddifyregistrationfieldsModel->values_req = $form_values['values_req'];
            $AddifyregistrationfieldsModel->users_to_edit = $form_values['users_to_edit'];
            $AddifyregistrationfieldsModel->display_msg = $form_values['display_msg'];
            $AddifyregistrationfieldsModel->enable_dependent_field = $form_values['enable_dependent_field'];
            $AddifyregistrationfieldsModel->dependent_value = $form_values['dependent_value'];
            $AddifyregistrationfieldsModel->date_update = $dt;
        } else {
            $AddifyregistrationfieldsModel = new AddifyregistrationfieldsModel();
            $AddifyregistrationfieldsModel->rule_active = $form_values['rule_active'];
            $AddifyregistrationfieldsModel->title = $form_values['title'];
            $AddifyregistrationfieldsModel->priority = $form_values['priority'];
            $AddifyregistrationfieldsModel->max_size = $form_values['max_size'];
            $AddifyregistrationfieldsModel->file_type = $form_values['file_type'];
            $AddifyregistrationfieldsModel->field_type = $form_values['field_type'];
            $AddifyregistrationfieldsModel->dependent_field = $form_values['dependent_field'];
            $AddifyregistrationfieldsModel->field_validation = $form_values['field_validation'];
            $AddifyregistrationfieldsModel->field_name = $form_values['field_name'];
            $AddifyregistrationfieldsModel->field_label = $form_values['field_label'];
            $AddifyregistrationfieldsModel->short_description = $form_values['short_description'];
            $AddifyregistrationfieldsModel->place_holder = $form_values['place_holder'];
            $AddifyregistrationfieldsModel->field_heading = $form_values['field_heading'];
            $AddifyregistrationfieldsModel->options = $form_values['options'];
            $AddifyregistrationfieldsModel->values_req = $form_values['values_req'];
            $AddifyregistrationfieldsModel->users_to_edit = $form_values['users_to_edit'];
            $AddifyregistrationfieldsModel->display_msg = $form_values['display_msg'];
            $AddifyregistrationfieldsModel->enable_dependent_field = $form_values['enable_dependent_field'];
            $AddifyregistrationfieldsModel->dependent_value = $form_values['dependent_value'];
            $AddifyregistrationfieldsModel->date_add = $dt;
        }
        if (!call_user_func(array($AddifyregistrationfieldsModel, $call_back))) {
            $this->context->controller->errors = sprintf($this->l('Something went wrong while performing operation %s'), $call_back);
        } else {
            $this->flushCache();
            if ($call_back == 'add') {
                Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&conf=3&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name . '&active_tab=2');
            } else {
                Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&conf=4&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name . '&active_tab=2');
            }
        }
    }
    /**
     * Create the structure of your form.
     */
    protected function getConfigForm()
    {
        $fields_form =
            array(
                'form' => array(
                    'legend' => array(
                        'title' => $this->l('Genaeral Settings'),
                        'icon' => 'icon-cogs',
                    ),
                    'input' => array(
                        array(
                            'type' => 'switch',
                            'label' => $this->l('Active Module'),
                            'name' => 'addifyregistrationfields_LIVE_MODE',
                            'desc' => $this->l('Enable/Disable this Module'),
                            'values' => array(
                                array(
                                    'id' => 'active_on',
                                    'value' => 1,
                                    'label' => $this->l('Enable')
                                ),
                                array(
                                    'id' => 'active_off',
                                    'value' => 0,
                                    'label' => $this->l('Disable')
                                ),
                            ),
                        ),
                        array(
                            'type' => 'switch',
                            'label' => $this->l('Show Fields on My Account'),
                            'name' => 'addifyregistrationfields_show_field_account',
                            'desc' => $this->l('Display separate section for registration fields in My account section'),
                            'values' => array(
                                array(
                                    'id' => 'active_on',
                                    'value' => 1,
                                    'label' => $this->l('Enable')
                                ),
                                array(
                                    'id' => 'active_off',
                                    'value' => 0,
                                    'label' => $this->l('Disable')
                                ),
                            ),
                        ),
                        array(
                            'type' => 'switch',
                            'label' => $this->l('Show Fields on Admin Customer Detail Page'),
                            'name' => 'addifyregistrationfields_show_field_admin',
                            'desc' => $this->l('Display registration fields in admin customer detail page.'),
                            'values' => array(
                                array(
                                    'id' => 'active_on',
                                    'value' => 1,
                                    'label' => $this->l('Enable')
                                ),
                                array(
                                    'id' => 'active_off',
                                    'value' => 0,
                                    'label' => $this->l('Disable')
                                ),
                            ),
                        ),
                    ),
                    'submit' => array(
                        'title' => $this->l('Save'),
                    ),
                )
            );
        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);
        $helper->identifier = $this->identifier;
        $helper->submit_action = "Submitconfig{$this->name}";
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false)
            . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFormValues(),  /* Add values for your inputs */
            // 'languages' => Language::getLanguages(),
            'id_language' => $this->context->language->id,
        );
        return $helper->generateForm(array($fields_form));
    }
    /**
     * Set values for the inputs.
     */
    protected function getConfigFormValues()
    {
        $fields_value = array();
        if ((Tools::isSubmit('Submitconfig' . $this->name) === false)) {
            $fields_value['addifyregistrationfields_LIVE_MODE'] = Configuration::get('addifyregistrationfields_LIVE_MODE');
            $fields_value['addifyregistrationfields_show_field_account'] = Configuration::get('addifyregistrationfields_show_field_account');
            $fields_value['addifyregistrationfields_show_field_admin'] = Configuration::get('addifyregistrationfields_show_field_admin');
        } else {
            $fields_value['addifyregistrationfields_LIVE_MODE'] = Tools::getValue('addifyregistrationfields_LIVE_MODE');
            $fields_value['addifyregistrationfields_show_field_account'] = Tools::getValue('addifyregistrationfields_show_field_account');
            $fields_value['addifyregistrationfields_show_field_admin'] = Tools::getValue('addifyregistrationfields_show_field_admin');
        }
        return $fields_value;
    }
    /**
     * Save form data.
     */
    protected function processConfigSave()
    {
        $form_values = $this->getConfigFormValues();
        Configuration::updateValue('addifyregistrationfields_LIVE_MODE', $form_values['addifyregistrationfields_LIVE_MODE']);
        Configuration::updateValue('addifyregistrationfields_show_field_account', $form_values['addifyregistrationfields_show_field_account']);
        Configuration::updateValue('addifyregistrationfields_show_field_admin', $form_values['addifyregistrationfields_show_field_admin']);
        $this->flushCache();
        Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&conf=4&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name . '&active_tab=1');
    }
    public function flushCache()
    {
        Tools::clearSmartyCache();
        Tools::clearXMLCache();
        Media::clearCache();
        // Tools::generateIndex();
        if (
            true === Tools::version_compare(_PS_VERSION_, '1.7.0.0', '>=') &&
            is_callable(array('Tools', 'clearAllCache'))
        ) {
            Tools::clearAllCache();
        }
    }
    public function hookDisplayAdminCustomers()
    {
        if (!$this->active) {
            return;
        }
        $addifyregistrationfields_LIVE_MODE = Configuration::get('addifyregistrationfields_LIVE_MODE');
        $addifyregistrationfields_show_field_admin = Configuration::get('addifyregistrationfields_show_field_admin');
        if ($addifyregistrationfields_LIVE_MODE == 1) {
            if ($addifyregistrationfields_show_field_admin == 1) {
                $context = Context::getContext();
                $getdatabycustomerid = AddifyregistrationfieldsdataModel::getDataByCustomerId((int) $context->customer->id);
                if ($getdatabycustomerid) {
                    $this->smarty->assign(array(
                        'databycustomerid' => $getdatabycustomerid,
                        'baseurl' => Context::getContext()->shop->getBaseURL(),
                    ));
                    $template = $this->display(__FILE__, '/views/templates/admin/arf_admin_field_data_template.tpl');
                    return $template;
                }
            }
        }
    }
    public function hookDisplayBackOfficeHeader()
    {
        if (!$this->active) {
            return;
        }
        $addifyregistrationfields_LIVE_MODE = Configuration::get('addifyregistrationfields_LIVE_MODE');
        if ($addifyregistrationfields_LIVE_MODE == 1) {
            $get_rules = AddifyregistrationfieldsModel::getRulesForFields();
            $dependent_value = '';
            $fields_dependent = array(
                array('id' => 'none', 'name' => $this->l('None'))
            );
            if (Tools::getValue('id')) {
                $AddifyregistrationfieldsModel = new AddifyregistrationfieldsModel((int) Tools::getValue('id'));
                $dependent_value = $AddifyregistrationfieldsModel->dependent_value;
                $get_saved_data = new AddifyregistrationfieldsModel((int) Tools::getValue('id'));
                foreach ($get_rules as $get_rule) {
                    if ($get_rule['field_type'] == "multiselect" || $get_rule['field_type'] == "select" || $get_rule['field_type'] == "checkbox" || $get_rule['field_type'] == "radio") {
                        if ($get_saved_data->id != $get_rule['id']) {
                            $fields_dependent[] = array('id' => $get_rule['id'], 'name' => $get_rule['id'] . '-' . $get_rule['field_name']);
                        }
                    }
                }
                $getRulesForFields = [];
                if ($get_rules) {
                    foreach ($get_rules as $get_rule) {
                        if ($get_rule['id'] == Tools::getValue('id')) {
                        } else {
                            $getRulesForFields[] = $get_rule;
                        }
                    }
                }
            } else {
                foreach ($get_rules as $get_rule) {
                    if ($get_rule['field_type'] == "multiselect" || $get_rule['field_type'] == "select" || $get_rule['field_type'] == "checkbox" || $get_rule['field_type'] == "radio") {
                        $fields_dependent[] = array('id' => $get_rule['id'], 'name' => $get_rule['id'] . '-' . $get_rule['field_name']);
                    }
                }
                $getRulesForFields = AddifyregistrationfieldsModel::getRulesForFields();
            }
            Media::addJsDef(array(
                "arf_get_rules" => $getRulesForFields,
                "dependent_value" => $dependent_value,
                "fields_dependent" => $fields_dependent,
            ));
            $this->context->controller->addJS($this->_path . 'views/js/arf_back_office.js');
        }
    }
    /**
     * Add the CSS & JavaScript files you want to be added on the FO.
     */
    public function hookDisplayHeader()
    {
        if (!$this->active) {
            return;
        }
        $addifyregistrationfields_LIVE_MODE = Configuration::get('addifyregistrationfields_LIVE_MODE');
        if ($addifyregistrationfields_LIVE_MODE == 1) {
            $type = Dispatcher::getInstance()->getController();
            // if ($type != "authentication" && $type != "identity" && $type != "registration") {
            //     $this->context->cookie->__unset('get_datas');
            // }
            $getrules = AddifyregistrationfieldsModel::getRules();
            $link = new Link();
            $arf_add_ajax_link = $link->getModuleLink($this->name, 'arfsavedata');
            if ($getrules) {
                // if (!$this->context->cookie->__isset('get_datas')) {
                //     Media::addJsDef(array(
                //         "get_datas" => '',
                //     ));
                // }
                Media::addJsDef(array(
                    "arf_add_ajax_controller_link" => $arf_add_ajax_link,
                    "getrules" => $getrules,
                ));
                $this->context->controller->addJS($this->_path . 'views/js/arf_custom_fields.js');
            }
        }
    }
    public function hookDisplayCustomerAccountForm()
    {
        if (!$this->active) {
            return;
        }
        $customerData = $this->context->customer;
        $customerData->isLogged();
        $type = Dispatcher::getInstance()->getController();
        $addifyregistrationfields_show_field_account = Configuration::get('addifyregistrationfields_show_field_account');
        if ($type == "identity") {
            if ($addifyregistrationfields_show_field_account == 0) {
                return;
            }
        }
        $addifyregistrationfields_LIVE_MODE = Configuration::get('addifyregistrationfields_LIVE_MODE');
        if ($addifyregistrationfields_LIVE_MODE == 1) {
            $context = Context::getContext();
            $getrules = AddifyregistrationfieldsModel::getRules();
            if ($getrules) {
                usort($getrules, function ($a, $b) {
                    return $a['sort_order'] <=> $b['sort_order'];
                });
            }
            $module = Module::getInstanceByName('addifyregistrationfields');
            $getRulesForFields = AddifyregistrationfieldsModel::getRulesForFields();
            $getdatabycustomerid = AddifyregistrationfieldsdataModel::getDataByCustomerId((int) $context->customer->id);
            $fields_dependent = [];
            foreach ($getRulesForFields as $get_rule) {
                if ($get_rule['field_type'] == "multiselect" || $get_rule['field_type'] == "select" || $get_rule['field_type'] == "checkbox" || $get_rule['field_type'] == "radio") {
                    $fields_dependent[] = array('id' => $get_rule['id'], 'name' => $get_rule['id'] . '-' . $get_rule['field_name']);
                }
            }
            $remain_fields = [];
            foreach ($fields_dependent as $key => $field_dependent) {
                $depend_flag = 0;
                foreach ($getrules as $key => $getrule) {
                    if ($field_dependent['id'] == $getrule['dependent_field']) {
                        $depend_flag = 1;
                    }
                }
                if ($depend_flag == 0) {
                    $remain_fields[] = $field_dependent['id'];
                }
            }
            $link = new Link();
            $arf_get_ajax_link = $link->getModuleLink($this->name, 'arfsavedata');
            if ($getrules) {
                $final_rules = [];
                foreach ($getrules as $getrule) {
                    $rule_flag = 1;
                    if ($final_rules) {
                        foreach ($final_rules as $final_rule) {
                            if ($final_rule['dependent_field'] == $getrule['dependent_field'] && $getrule['dependent_field'] != "none") {
                                $rule_flag = 0;
                            }
                        }
                    }
                    if ($rule_flag == 1) {
                        $final_rules[] = $getrule;
                    }
                }
                $this->context->smarty->assign(array(
                    'final_rules' => $final_rules,
                    'rules_data' => $getrules,
                    'remain_fields' => $remain_fields,
                    'arf_get_ajax_link' => $arf_get_ajax_link,
                    'db_datas' => $getdatabycustomerid,
                    'id_module' => $module->id,
                    'customer_loggedin' => $customerData->isLogged()
                ));
                return $this->display(__FILE__, '/views/templates/front/display_registration_fields.tpl');
            }
        }
    }
    public function hookActionSubmitAccountBefore()
    {
            if (!$this->active) {
                return;
            }
            if (Configuration::get('addifyregistrationfields_LIVE_MODE') == 0) {
                return;
            }
            // $getrules = AddifyregistrationfieldsModel::getRules();
            // $module = Module::getInstanceByName('addifyregistrationfields');
            // $link = new Link();
            // $arf_get_ajax_link = $link->getModuleLink($this->name, 'arfsavedata');
            // if ($getrules) {
            //     $get_datas = $this->checkValidation($getrules);
            //     if (isset($get_datas) && $get_datas) {
            //         $count = 0;
            //         $flag = 0;
            //         foreach ($get_datas as $get_data) {
            //             if ($get_data['required'] == 0 && $get_data['validate'] == 0) {
            //                 $count = $count + 1;
            //             } else {
            //                 if ($get_data['field_type'] == 'message') {
            //                     $count = $count + 1;
            //                 }
            //             }
            //         }
            //         if ($count == count($get_datas)) {
            //             $flag = 1;
            //         }
            //         if ($flag == 0) {
            //             $this->context->cookie->__set('get_datas', '1');
            //             Media::addJsDef(array(
            //                 "get_datas" => $get_datas,
            //             ));
            //             return false;
            //         }
            //     }
            // }
            return true;
    }
    // public static function checkValidation($all_rules)
    // {
    //     $result_array_final = [];
    //     foreach ($all_rules as $key => $getrule) {
    //         $result_array = [];
    //         $enable_dependent_field = $getrule['enable_dependent_field'];
    //         $field_type = $getrule['field_type'];
    //         $field_name = $getrule['field_name'];
    //         $rule_id = $getrule['id'];
    //         $final_field_name = $field_name . $rule_id;
    //         $rule_file_size = $getrule['max_size'];
    //         $rule_file_type = $getrule['file_type'];
    //         $error_id = "error" . $field_name . $getrule['id'];
    //         $result_array['field_name'] = $field_name;
    //         $result_array['rule_id'] = $rule_id;
    //         $result_array['field_type'] = $field_type;
    //         $result_array['error_id'] = $error_id;
    //         $result_array_final[$key] = $result_array;
    //         $field_data = Tools::getValue($final_field_name);
    //         if ($getrule['values_req'] == 1) {
    //             $flag_req = 0;
    //             if ($enable_dependent_field == 1) {
    //                 $dependent_rule_id = $getrule['dependent_field'];
    //                 $dependent_value = $getrule['dependent_value'];
    //                 $getRuleById = AddifyregistrationfieldsModel::getRuleById((int) $dependent_rule_id);
    //                 if ($getRuleById) {
    //                     $final_field_name = $getRuleById[0]['field_name'] . $getRuleById[0]['id'];
    //                     $field_value = Tools::getValue($final_field_name);
    //                     if ($field_value == $dependent_value) {
    //                         $flag_req = 1;
    //                     }
    //                 } else {
    //                     if ($dependent_rule_id === 'none') {
    //                         $flag_req = 1;
    //                     }
    //                 }
    //             } else {
    //                 $flag_req = 1;
    //             }
    //             if ($flag_req == 1) {
    //                 if ($field_type == 'fileupload') {
    //                     if ($_FILES[$final_field_name]['name'] == "") {
    //                         $result_array['required'] = "1";
    //                         $result_array_final[$key] = $result_array;
    //                     } else {
    //                         $result_array['required'] = "0";
    //                         $result_array_final[$key] = $result_array;
    //                     }
    //                 } else {
    //                     if (empty($field_data)) {
    //                         $result_array['required'] = "1";
    //                         $result_array_final[$key] = $result_array;
    //                     } else {
    //                         $result_array['required'] = "0";
    //                         $result_array_final[$key] = $result_array;
    //                     }
    //                 }
    //             } else {
    //                 $result_array['required'] = "0";
    //                 $result_array_final[$key] = $result_array;
    //             }
    //         }
    //         if ($getrule['values_req'] == 0) {
    //             $result_array['required'] = "0";
    //             $result_array_final[$key] = $result_array;
    //         }
    //         if ($getrule['field_type'] == "number") {
    //             if ($getrule['field_validation'] == "isInt") {
    //                 if (!empty($field_data)) {
    //                     if (!Validate::isInt($field_data)) {
    //                         $result_array['validate'] = "1";
    //                         $result_array_final[$key] = $result_array;
    //                     } else {
    //                         $result_array['validate'] = "0";
    //                         $result_array_final[$key] = $result_array;
    //                     }
    //                 } else {
    //                     $result_array['validate'] = "0";
    //                     $result_array_final[$key] = $result_array;
    //                 }
    //             } else if ($getrule['field_validation'] == "isFloat") {
    //                 if (!empty($field_data)) {
    //                     if (!preg_match('/^[-+]?[0-9]+\.[0-9]+$/', $field_data)) {
    //                         $result_array['validate'] = "1";
    //                         $result_array_final[$key] = $result_array;
    //                     } else {
    //                         $result_array['validate'] = "0";
    //                         $result_array_final[$key] = $result_array;
    //                     }
    //                 } else {
    //                     $result_array['validate'] = "0";
    //                     $result_array_final[$key] = $result_array;
    //                 }
    //             } else {
    //                 $result_array['validate'] = "0";
    //                 $result_array_final[$key] = $result_array;
    //             }
    //         } else if ($getrule['field_type'] == "date") {
    //             if ($getrule['field_validation'] == "isDate") {
    //                 if (!empty($field_data)) {
    //                     if (!Validate::isDate($field_data)) {
    //                         $result_array['validate'] = "1";
    //                         $result_array_final[$key] = $result_array;
    //                     } else {
    //                         $result_array['validate'] = "0";
    //                         $result_array_final[$key] = $result_array;
    //                     }
    //                 } else {
    //                     $result_array['validate'] = "0";
    //                     $result_array_final[$key] = $result_array;
    //                 }
    //             } else {
    //                 $result_array['validate'] = "0";
    //                 $result_array_final[$key] = $result_array;
    //             }
    //         } else if ($getrule['field_type'] == "email") {
    //             if ($getrule['field_validation'] == "isEmail") {
    //                 if (!empty($field_data)) {
    //                     if (!Validate::isEmail($field_data)) {
    //                         $result_array['validate'] = "1";
    //                         $result_array_final[$key] = $result_array;
    //                     } else {
    //                         $result_array['validate'] = "0";
    //                         $result_array_final[$key] = $result_array;
    //                     }
    //                 } else {
    //                     $result_array['validate'] = "0";
    //                     $result_array_final[$key] = $result_array;
    //                 }
    //             } else {
    //                 $result_array['validate'] = "0";
    //                 $result_array_final[$key] = $result_array;
    //             }
    //         } else if ($getrule['field_type'] == "text") {
    //             if ($getrule['field_validation'] == "isString") {
    //                 if (!empty($field_data)) {
    //                     if (!Validate::isString($field_data)) {
    //                         $result_array['validate'] = "1";
    //                         $result_array_final[$key] = $result_array;
    //                     } else {
    //                         $result_array['validate'] = "0";
    //                         $result_array_final[$key] = $result_array;
    //                     }
    //                 } else {
    //                     $result_array['validate'] = "0";
    //                     $result_array_final[$key] = $result_array;
    //                 }
    //             } else if ($getrule['field_validation'] == "isGenericName") {
    //                 if (!empty($field_data)) {
    //                     if (!preg_match('/^([a-zA-Z]+)$/', $field_data)) {
    //                         $result_array['validate'] = "1";
    //                         $result_array_final[$key] = $result_array;
    //                     } else {
    //                         $result_array['validate'] = "0";
    //                         $result_array_final[$key] = $result_array;
    //                     }
    //                 } else {
    //                     $result_array['validate'] = "0";
    //                     $result_array_final[$key] = $result_array;
    //                 }
    //             } else if ($getrule['field_validation'] == "isUrl") {
    //                 if (!empty($field_data)) {
    //                     if (!preg_match("/\b(?:(?:https?|ftp):\/\/|www\.)[-a-z0-9+&@#\/%?=~_|!:,.;]*[-a-z0-9+&@#\/%=~_|]/i", $field_data)) {
    //                         $result_array['validate'] = "1";
    //                         $result_array_final[$key] = $result_array;
    //                     } else {
    //                         $result_array['validate'] = "0";
    //                         $result_array_final[$key] = $result_array;
    //                     }
    //                 } else {
    //                     $result_array['validate'] = "0";
    //                     $result_array_final[$key] = $result_array;
    //                 }
    //             } else {
    //                 $result_array['validate'] = "0";
    //                 $result_array_final[$key] = $result_array;
    //             }
    //         } else if ($getrule['field_type'] == "vat_number") {
    //             if ($getrule['field_validation'] == "isVAT") {
    //                 if (!empty($field_data)) {
    //                     $vatid = $field_data;
    //                     $countryCode = Tools::substr($vatid, 0, 2);
    //                     $vatNumber = Tools::substr($vatid, 2);
    //                     //Prepare the URL
    //                     $url = 'https://ec.europa.eu/taxation_customs/vies/viesquer.do?selectedLanguage=PL&ms=' . $countryCode . '&iso=' . $countryCode . '&vat=' . $vatNumber . '&name=&companyType=&street1=&postcode=&city=&requesterMs=&requesterIso=&requesterVat=';
    //                     $geturlcontent = Tools::file_get_contents($url);
    //                     $part = Tools::substr($geturlcontent, 12411, 100);
    //                     if (strpos($part, 'Yes') !== false) {
    //                         $result_array['validate'] = "0";
    //                         $result_array_final[$key] = $result_array;
    //                     } else {
    //                         $result_array['validate'] = "1";
    //                         $result_array_final[$key] = $result_array;
    //                     }
    //                 } else {
    //                     $result_array['validate'] = "0";
    //                     $result_array_final[$key] = $result_array;
    //                 }
    //             } else {
    //                 $result_array['validate'] = "0";
    //                 $result_array_final[$key] = $result_array;
    //             }
    //         } else if ($getrule['field_type'] == "fileupload") {
    //             if (isset($_FILES[$final_field_name]) == true) {
    //                 if ($_FILES[$final_field_name]['name'] != "") {
    //                     $target_dir = _PS_MODULE_DIR_ . 'addifyregistrationfields/views/uploads/';
    //                     $without_extension = Tools::substr(basename($_FILES[$final_field_name]["name"]), 0, strrpos(basename($_FILES[$final_field_name]["name"]), "."));
    //                     $ext = Tools::substr(basename($_FILES[$final_field_name]["name"]), strrpos(basename($_FILES[$final_field_name]["name"]), '.'));
    //                     $t = time();
    //                     $date_key = ($t) . (date("Y-m-d", $t));
    //                     $target_file = $target_dir . $without_extension . $date_key . $ext;
    //                     $uploadOk = 1;
    //                     $imageFileType = pathinfo($target_file, PATHINFO_EXTENSION);
    //                     // Check if file already exists
    //                     if (file_exists($target_file)) {
    //                         $result_array['validate_exists'] = "1";
    //                         $result_array_final[$key] = $result_array;
    //                         $uploadOk = 0;
    //                     }
    //                     // Check file size
    //                     if ($rule_file_size) {
    //                         $final_rule_file_size = $rule_file_size * 1024 * 1024;
    //                         if ($_FILES[$final_field_name]["size"] > $final_rule_file_size) {
    //                             $result_array['validate_size'] = "1";
    //                             $result_array_final[$key] = $result_array;
    //                             $uploadOk = 0;
    //                         }
    //                     }
    //                     // Allow certain file formats
    //                     if ($rule_file_type) {
    //                         $rulefiletypes = explode(',', $rule_file_type);
    //                         if (!in_array($imageFileType, $rulefiletypes)) {
    //                             $result_array['validate_format'] = "1";
    //                             $result_array_final[$key] = $result_array;
    //                             $uploadOk = 0;
    //                         }
    //                     }
    //                     // Check if $uploadOk is set to 0 by an error
    //                     if ($uploadOk == 0) {
    //                         $result_array['validate'] = "1";
    //                         $result_array_final[$key] = $result_array;
    //                     }
    //                     // if everything is ok, try to upload file
    //                     else {
    //                         $result_array['validate'] = "0";
    //                         $result_array_final[$key] = $result_array;
    //                     }
    //                 } else {
    //                     // $result_array['validate'] = "1";
    //                     $result_array['validate'] = "0";
    //                     $result_array_final[$key] = $result_array;
    //                 }
    //             } else {
    //                 $result_array['validate'] = "0";
    //                 $result_array_final[$key] = $result_array;
    //             }
    //         } else {
    //             $result_array['validate'] = "0";
    //             $result_array_final[$key] = $result_array;
    //         }
    //     }
    //     return $result_array_final;
    // }
    public function hookActionCustomerAccountAdd()
    {
        if (!$this->active) {
            return;
        }
        if (Configuration::get('addifyregistrationfields_LIVE_MODE') == 0) {
            return;
        }
        $getrules = AddifyregistrationfieldsdataModel::getDataForCustomer();
        $context = Context::getContext();
        $id_customer = (int) $context->customer->id;
        if ($getrules) {
            foreach ($getrules as $key => $rules_data) {
                if ($rules_data['id_customer'] == 0) {
                    $addifyregistrationfieldsdata = new AddifyregistrationfieldsdataModel($rules_data['id_data']);
                    $addifyregistrationfieldsdata->id_customer = $id_customer;
                    $addifyregistrationfieldsdata->save();
                }
            }
        }
    }



    public function hookactionObjectCustomerUpdateBefore($params)
    {
        //     if (!$this->active) {
        //         return;
        //     }
        //     if (Configuration::get('addifyregistrationfields_LIVE_MODE') == 0) {
        //         return;
        //     }
        //     $getrules = AddifyregistrationfieldsModel::getRules();
        //     $module = Module::getInstanceByName('addifyregistrationfields');
        //     $link = new Link();
        //     $arf_get_ajax_link = $link->getModuleLink($this->name, 'arfsavedata');
        //     if ($getrules) {
        //         $get_datas = $this->checkValidation($getrules);
        //         if (isset($get_datas) && $get_datas) {
        //             $count = 0;
        //             $flag = 0;
        //             foreach ($get_datas as $get_data) {
        //                 if ($get_data['required'] == 0 && $get_data['validate'] == 0) {
        //                     $count = $count + 1;
        //                 } else {
        //                     if ($get_data['field_type'] == 'message') {
        //                         $count = $count + 1;
        //                     }
        //                 }
        //             }
        //             if ($count == count($get_datas)) {
        //                 $flag = 1;
        //             }
        //             if ($flag == 0) {
        //                 $this->context->cookie->__set('get_datas', '1');
        //                 Media::addJsDef(array(
        //                     "get_datas" => $get_datas,
        //                 ));
        //                 return false;
        //             }
        //         }
        //     }
        //     return true;
    }
    public function hookActionCustomerAccountUpdate($params)
    {
        //     if (!$this->active) {
        //         return;
        //     }
        //     if (Configuration::get('addifyregistrationfields_LIVE_MODE') == 0) {
        //         return;
        //     }
        //     $getrules = AddifyregistrationfieldsModel::getRules();
        //     $context = Context::getContext();
        //     if ($getrules) {
        //         foreach ($getrules as $all_field_data) {
        //             $id_customer = (int) $context->customer->id;
        //             $field_type = $all_field_data['field_type'];
        //             $field_name = $all_field_data['field_name'];
        //             $id_rule = $all_field_data['id'];
        //             $dependent_field = $all_field_data['dependent_field'];
        //             $dependent_value = $all_field_data['dependent_value'];
        //             $enable_dependent_field = $all_field_data['enable_dependent_field'];
        //             $id_field = $field_name . $id_rule;
        //             $field_label = $all_field_data['field_label'];
        //             $field_options = '';
        //             $field_value = '';
        //             if (
        //                 $field_type == 'message' || $field_type == 'heading' ||
        //                 $field_type == 'text' || $field_type == 'vat_number' || $field_type == 'number' || $field_type == 'email' || $field_type == 'date' || $field_type == 'time' ||
        //                 $field_type == 'color' || $field_type == 'textarea' || $field_type == 'fileupload' || $field_type == 'switch' || $field_type == 'select' || $field_type == 'radio'
        //             ) {
        //                 if ($field_type == 'fileupload') {
        //                     if (isset($_FILES[$id_field]) == true) {
        //                         if ($_FILES[$id_field]['name'] != "") {
        //                             $target_dir = _PS_MODULE_DIR_ . 'addifyregistrationfields/views/uploads/';
        //                             $without_extension = Tools::substr(basename($_FILES[$id_field]["name"]), 0, strrpos(basename($_FILES[$id_field]["name"]), "."));
        //                             $ext = Tools::substr(basename($_FILES[$id_field]["name"]), strrpos(basename($_FILES[$id_field]["name"]), '.'));
        //                             $t = time();
        //                             $date_key = ($t) . (date("Y-m-d", $t));
        //                             $target_file = $target_dir . $without_extension . $date_key . $ext;
        //                             // if everything is ok, try to upload file
        //                             if (move_uploaded_file($_FILES[$id_field]["tmp_name"], $target_file)) {
        //                                 $field_value = $without_extension . $date_key . $ext;
        //                                 $field_options = '';
        //                             }
        //                         }
        //                     } else {
        //                         $field_value = Tools::getValue($id_field);
        //                         $field_options = '';
        //                     }
        //                 } else {
        //                     $field_value = Tools::getValue($id_field);
        //                     $field_options = '';
        //                 }
        //             } else {
        //                 if (!Tools::getValue($id_field)) {
        //                     $field_value = '';
        //                     $field_options = Tools::getValue($id_field);
        //                 } else {
        //                     $field_value = '';
        //                     $field_options = implode(",", Tools::getValue($id_field));
        //                 }
        //             }
        //             $arf_checkdata = AddifyregistrationfieldsdataModel::checkData($id_customer, $id_field, $id_rule);
        //             if ($arf_checkdata) {
        //                 $AddifyregistrationfieldsdataModel = new AddifyregistrationfieldsdataModel((int) $arf_checkdata[0]['id_data']);
        //             } else {
        //                 $AddifyregistrationfieldsdataModel = new AddifyregistrationfieldsdataModel();
        //             }
        //             $AddifyregistrationfieldsdataModel->id_customer = (int) $id_customer;
        //             $AddifyregistrationfieldsdataModel->id_field = pSQL($id_field);
        //             $AddifyregistrationfieldsdataModel->id_rule = (int) $id_rule;
        //             $AddifyregistrationfieldsdataModel->dependent_field = (int) $dependent_field;
        //             $AddifyregistrationfieldsdataModel->enable_dependent_field = (int) $enable_dependent_field;
        //             $AddifyregistrationfieldsdataModel->dependent_value = pSQL($dependent_value);
        //             $AddifyregistrationfieldsdataModel->field_label = pSQL($field_label);
        //             $AddifyregistrationfieldsdataModel->field_type = pSQL($field_type);
        //             $AddifyregistrationfieldsdataModel->field_options = pSQL($field_options);
        //             $AddifyregistrationfieldsdataModel->data__for_options = pSQL($all_field_data['options']);
        //             $AddifyregistrationfieldsdataModel->field_name = pSQL($field_name);
        //             $AddifyregistrationfieldsdataModel->field_value = pSQL($field_value);
        //             $AddifyregistrationfieldsdataModel->save();
        //         }
        //     }
    }
    public function hookDisplayGDPRConsent()
    {
    }
    public function hookRegisterGDPRConsent()
    {
    }
    public function hookActionDeleteGDPRCustomer($customer)
    {
        if (!empty($customer['id']) && Validate::isEmail($customer['email'])) {
            $sql = "DELETE FROM " . _DB_PREFIX_ . "addifyregistrationfieldsdata WHERE id_customer = '" . (int) $customer['id'] . "'";
            if (Db::getInstance()->execute($sql)) {
                return json_encode(true);
            }
            return json_encode($this->l('Addifyregistrationfields : Unable to delete customer using email.'));
        }
    }
    public function hookActionExportGDPRData($customer)
    {
        if (!Tools::isEmpty($customer['id']) && Validate::isEmail($customer['email'])) {
            $sql = "SELECT field_name AS `Field Name`,field_options AS `Field Options`,field_value AS `Field Value` FROM " . _DB_PREFIX_ . "addifyregistrationfieldsdata WHERE id_customer = '" . (int) $customer['id'] . "'";
            if ($res = Db::getInstance()->ExecuteS($sql)) {
                return json_encode($res);
            }
            return json_encode($this->l('Addifyregistrationfields : Unable to export customer using email.'));
        }
    }
}